//
//  ISNetworking.m
//  ImageSharing
//
//  Created by Daniel Gur on 9/25/13.
//  Copyright (c) 2013 P2P. All rights reserved.
//

#import "ISNetworking.h"
#import "GCDAsyncSocket.h"

#define WELCOME_MSG  0
#define ECHO_MSG     1
#define WARNING_MSG  2

#define READ_TIMEOUT 15.0
#define READ_TIMEOUT_EXTENSION 10.0

#pragma mark -


@implementation ISNetworking

- (id)init {
    if (self = [super init]) {
        socketQueue = dispatch_queue_create("socketQueue", NULL);
        listenSocket = [[GCDAsyncSocket alloc] initWithDelegate:self delegateQueue:socketQueue];
        connectedSockets = [[NSMutableArray alloc] initWithCapacity:1];
        isRunning = NO;
    }
	return self;
}

- (void)startStop {
	if(!isRunning) {
		int port = 1234;
		
		// ensure (port > 0 && port < 65535)
		
		NSError *error = nil;
		if(![listenSocket acceptOnPort:port error:&error]) {
			NSLog(@"Error starting server: %@", error);
			return;
		}
		
		NSLog(@"Echo server started on port %hu", [listenSocket localPort]);
		isRunning = YES;
	} else {
		[listenSocket disconnect];
		
		@synchronized(connectedSockets) {
			for (NSUInteger i = 0; i < [connectedSockets count]; i++) {
				[[connectedSockets objectAtIndex:i] disconnect];
			}
		}
		
        NSLog(@"Stopped Echo server");
		isRunning = false;
	}
}

- (void)socket:(GCDAsyncSocket *)sock didAcceptNewSocket:(GCDAsyncSocket *)newSocket {
	@synchronized(connectedSockets) {
		[connectedSockets addObject:newSocket];
	}
	
	NSString *host = [newSocket connectedHost];
	UInt16 port = [newSocket connectedPort];
	
    NSLog(@"New host added %@", host);
	dispatch_async(dispatch_get_main_queue(), ^{
		@autoreleasepool {
			NSLog(@"Accepted client %@:%hu", host, port);
		}
	});
	
	NSString *welcomeMsg = @"Welcome to the AsyncSocket Echo Server\r\n";
	NSData *welcomeData = [welcomeMsg dataUsingEncoding:NSUTF8StringEncoding];
	
	[newSocket writeData:welcomeData withTimeout:-1 tag:WELCOME_MSG];
	
	[newSocket readDataToData:[GCDAsyncSocket CRLFData] withTimeout:READ_TIMEOUT tag:0];
}

- (void)socket:(GCDAsyncSocket *)sock didWriteDataWithTag:(long)tag {
	if (tag == ECHO_MSG) {
		[sock readDataToData:[GCDAsyncSocket CRLFData] withTimeout:READ_TIMEOUT tag:0];
	}
}

- (void)socket:(GCDAsyncSocket *)sock didReadData:(NSData *)data withTag:(long)tag {
	
	dispatch_async(dispatch_get_main_queue(), ^{
		@autoreleasepool {
            
			NSData *strData = [data subdataWithRange:NSMakeRange(0, [data length] - 2)];
			NSString *msg = [[NSString alloc] initWithData:strData encoding:NSUTF8StringEncoding];
			if (msg) {
				NSLog(@"%@", msg);
			} else {
				NSLog(@"Error converting received data into UTF-8 String");
			}
            
		}
	});
	
	// Echo message back to client
	[sock writeData:data withTimeout:-1 tag:ECHO_MSG];
}

/**
 * This method is called if a read has timed out.
 * It allows us to optionally extend the timeout.
 * We use this method to issue a warning to the user prior to disconnecting them.
 **/
- (NSTimeInterval)socket:(GCDAsyncSocket *)sock shouldTimeoutReadWithTag:(long)tag
                 elapsed:(NSTimeInterval)elapsed
               bytesDone:(NSUInteger)length {
	
    if (elapsed <= READ_TIMEOUT) {
		NSString *warningMsg = @"Are you still there?\r\n";
		NSData *warningData = [warningMsg dataUsingEncoding:NSUTF8StringEncoding];
		
		[sock writeData:warningData withTimeout:-1 tag:WARNING_MSG];
		
		return READ_TIMEOUT_EXTENSION;
	}
	
	return 0.0;
}

- (void)socketDidDisconnect:(GCDAsyncSocket *)sock withError:(NSError *)err {
	if (sock != listenSocket) {
		dispatch_async(dispatch_get_main_queue(), ^{
			@autoreleasepool {
				NSLog(@"Client Disconnected");
			}
		});
		
		@synchronized(connectedSockets) {
			[connectedSockets removeObject:sock];
		}
	}
}


@end
